#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (c) Camille Scott, 2020
# File   : zvars.py
# License: MIT
# Author : Camille Scott <camille.scott.w@gmail.com>
# Date   : 22.02.2021

from discord.ext import commands

from ..database import fetch_guild_db
from ..logging import LoggingMixin


class VarCommands(commands.Cog, LoggingMixin):

    def __init__(self, bot, db):
        self.bot = bot
        self.db = db

        super().__init__()

    @commands.group(name='var')
    async def var(self, ctx):
        '''
        Manage variables for the server.
        '''
        if ctx.invoked_subcommand is None:
            pass

    @var.command(name='list')
    @fetch_guild_db
    async def var_list(self, ctx):
        '''
        List currently set variables and values.
        '''
        variables = await ctx.guild_db.get_guild_vars()
        if variables:
            result = [f'**{key}**: {val}' for key, val in variables.items()]
            await ctx.send('\n'.join(result))
        else:
            await ctx.send('**No variables set.**')
    
    @var.command(name='set')
    @fetch_guild_db
    async def var_set(self, ctx, var: str, val: int):
        '''
        Set a variable.
        '''
        await ctx.guild_db.set_guild_var(ctx.author.id, var, val)
        await ctx.send(f'**{var}** = {val}')

    @var.command(name='get')
    @fetch_guild_db
    async def var_get(self, ctx, var: str):
        '''
        Get the given variable's value.
        '''
        val = await ctx.guild_db.get_guild_var(var)
        if val is None:
            await ctx.send(f'**{var}** is not defined.')
        else:
            await ctx.send(f'**{var}** = {val}')

    @var.command(name='del')
    @fetch_guild_db
    async def var_del(self, ctx, var: str):
        '''
        Delete the given variable.
        '''
        await ctx.guild_db.del_guild_var(var)
        await ctx.send(f'**{var}** deleted')



from __future__ import absolute_import, unicode_literals, division

from pyparsing import Word, opAssoc
from dice.elements import Integer, Roll, WildRoll, ExplodedRoll
from dice.exceptions import DiceException, DiceFatalException
from dice import roll, roll_min, roll_max, grammar
from pytest import raises


class TestInteger(object):
    def test_value(self):
        assert roll("1337") == 1337

    def test_type(self):
        assert isinstance(roll("1"), int)
        assert isinstance(roll("1"), Integer)


class TestDice(object):
    def test_dice_value(self):
        assert 0 < int(roll("d6")) <= 6
        assert 0 < int(roll("6d6")) <= 36

        assert 0 < int(roll("d%")) <= 100
        assert 0 < int(roll("6d%")) <= 600

        assert -1 <= int(roll("dF")) <= 1
        assert -6 <= int(roll("6dF")) <= 6
        assert -6 <= int(roll("u6")) <= 6
        assert -36 < int(roll("6u6")) <= 36

        assert 0 <= int(roll("w6"))
        assert 0 <= int(roll("6w6"))

        assert 0 < int(roll("d6x"))
        assert 0 < int(roll("6d6x"))

    def test_dice_type(self):
        assert isinstance(roll("d6"), Roll)
        assert isinstance(roll("6d6"), Roll)
        assert isinstance(roll("d%"), Roll)
        assert isinstance(roll("6d%"), Roll)

        assert isinstance(roll("dF"), Roll)
        assert isinstance(roll("6dF"), Roll)
        assert isinstance(roll("u6"), Roll)
        assert isinstance(roll("6u6"), Roll)

        assert isinstance(roll("w6"), WildRoll)
        assert isinstance(roll("6w6"), WildRoll)

        assert isinstance(roll("d6x"), ExplodedRoll)
        assert isinstance(roll("6d6x"), ExplodedRoll)

    def test_dice_values(self):
        for die in roll("6d6"):
            assert 0 < die <= 6

    def test_nested_dice(self):
        assert 1 <= roll("d(d6)t") <= 6
        assert -6 <= roll("u(d6)t") <= 6


class TestOperators(object):
    def test_add(self):
        assert roll("2 + 2") == 4

    def test_sub(self):
        assert roll("2 - 2") == 0

    def test_mul(self):
        assert roll("2 * 2") == 4
        assert roll("1d6 * 2") % 2 == 0

    def test_div(self):
        assert roll("2 / 2") == 1

    def test_mod(self):
        assert roll("5 % 3") == 2
        assert roll("1 % 3") == 1
        assert 0 <= roll("d6 % 3") <= 2

    def test_identity(self):
        assert roll("+2") == 2
        assert roll("+(1, 2)") == [1, 2]

    def test_negate(self):
        assert roll("-2") == -2
        assert roll("-(1, 2)") == [-1, -2]

    def test_aeso(self):
        assert roll("+-1") == -1
        assert roll("+-2") == 2
        assert roll("+-(1, 2)") == [-1, 2]


class TestVectorOperators(object):
    def test_total(self):
        assert (6 * 1) <= roll("6d6t") <= (6 * 6)

    def test_sort(self):
        value = roll("6d6s")
        assert value == sorted(value)
        assert isinstance(value, Roll)

        with raises(DiceFatalException):
            roll("6s")

    def test_drop(self):
        value = roll("6d6 v 3")
        assert len(value) == 3

        value = roll("(1, 2, 5, 9, 3) v 3")
        assert set(value) == set([1, 2, 3])

        value = roll("6d6v")
        assert len(value) == 5

        value = roll("6d6v(-3)")
        assert len(value) == 3

        with raises(DiceFatalException):
            roll("6 v 3")

    def test_keep(self):
        value = roll("6d6 ^ 3")
        assert len(value) == 3

        value = roll("(1, 2, 5, 9, 3) ^ 3")
        assert set(value) == set([3, 5, 9])

        value = roll("6d6^")
        assert len(value) == 5

        value = roll("6d6^(-3)")
        assert len(value) == 3

        with raises(DiceFatalException):
            roll("6 ^ 3")

    def test_middle(self):
        value = roll("6d6 o 3")
        assert len(value) == 3

        value = roll("(1, 2, 5, 9, 3) o 3")
        assert set(value) == set([2, 3, 5])

        value = roll("6d6o")
        assert len(value) == 4

        with raises(DiceFatalException):
            roll("6 o 3")

        assert len(roll("6d6o(-4)")) == 2

    def test_successes(self):
        assert roll("(2, 4, 6, 8) e 5") == 2
        assert roll("6 e 5") == 1
        assert roll("100d20e1") == 100
        with raises(DiceFatalException):
            roll("d20 e 21")
        with raises(DiceFatalException):
            roll("d(d6) e 6")

    def test_successe_failures(self):
        assert roll("(1, 2, 4, 6, 8) f 5") == 1
        assert roll("6 f 5") == 1
        assert roll("100d20f1") == 100
        with raises(DiceFatalException):
            roll("d20 f 21")
        with raises(DiceFatalException):
            roll("d(d6) f 6")

    def test_array_add(self):
        assert roll("(2, 4, 6, 8) .+ 2") == [4, 6, 8, 10]

    def test_array_sub(self):
        assert roll("(2, 4, 6, 8) .- 2") == [0, 2, 4, 6]

    def test_array(self):
        rr = roll("2d6, 3d6, 4d6")
        assert len(rr) == 3

    def test_extend(self):
        rr = roll("2d6 | 3d6, 4d6")
        assert len(rr) == 4

        rr2 = roll("2d6 | 3d6 | 4d6")
        assert len(rr2) == 9

        rr3 = roll("2d6 | 3d6 | 10 | 4d6")
        assert len(rr3) == 10


class TestDiceOperators(object):
    def test_reroll(self):
        r = roll("6d6r")
        assert len(r) == 6

    def test_force_reroll(self):
        r2 = roll("1000d6rr")
        assert 1 not in r2

        r3 = roll("100d6rr5")
        assert all(x > 5 for x in r3)

    def test_extreme_reroll(self):
        r = roll_min("2d6r")
        assert r == [1, 1]

        r = roll_max("2d6r6")
        assert r == [6, 6]

    def test_again_roll(self):
        while True:
            r = roll("6d6a")
            if 6 in r:
                break

        num_6 = r.count(6)
        assert 6 == len(r) - (num_6 // 2)

    def test_again_scalar(self):
        assert roll("6a6") == [6, 6]

    def test_again_vector(self):
        assert roll("(1,2,3)a2") == [1, 2, 2, 3]
        assert roll("(1|1|1)a1") == [1, 1, 1, 1, 1, 1]


class TestErrors(object):
    @staticmethod
    def run_test(expr):
        with raises((DiceException, DiceFatalException)):
            roll(expr)

    def test_bad_operators(self):
        for expr in ("6d", "1+", "[1,2,3]", "f", "3f", "3x", "6.+6", "7.-7"):
            self.run_test(expr)

    def test_invalid_rolls(self):
        for expr in ("(0-1)d6", "6d(0-1)", "d0", "6d0"):
            self.run_test(expr)

    def test_unmatched_parenthesis(self):
        for expr in ("(6d6", "6d6)"):
            self.run_test(expr)

    def test_explode_min(self):
        self.run_test("6d6x1")
        self.run_test("6d6x(1-2)")

    def test_explode_onesided(self):
        self.run_test("6d1x")

    def test_invalid_reroll(self):
        for expr in ("6r", "(1,2)r", "6rr", "(1,2)rr", "d(d6)rr", "u(d6)r"):
            self.run_test(expr)

    def test_div_zero(self):
        self.run_test("1/0")
        self.run_test("1/(0*1)")

    def test_again_fail(self):
        self.run_test("(1,2,3)a")
        self.run_test("1a")


class TestOperatorPrecedence(object):
    def test_operator_precedence_1(self):
        assert roll("16 / 8 * 4 + 2 - 1") == 9

    def test_operator_precedence_2(self):
        assert roll("16 - 8 + 4 * 2 / 1") == 16

    def test_operator_precedence_3(self):
        assert roll("10 - 3 + 2") == 9

    def test_operator_precedence_4(self):
        assert roll("1 + 2 * 3") == 7


class TestExpression(object):
    def test_expression(self):
        assert isinstance(roll("2d6"), Roll)

    def test_sub_expression(self):
        assert isinstance(roll("(2d6)d(2d6)"), Roll)


class TestBadPrecedence(object):
    def test_invalid_arity(self):
        with raises(Exception):
            grammar.operatorPrecedence(
                grammar.integer, [(Word("0"), 3, opAssoc.LEFT, Integer.parse)]
            )

    def test_invalid_association(self):
        with raises(Exception):
            grammar.operatorPrecedence(
                grammar.integer, [(Word("0"), 2, None, Integer.parse)]
            )



"""A library for parsing and evaluating dice notation."""

from __future__ import absolute_import, print_function, unicode_literals

from pyparsing import ParseBaseException

from . import elements
from . import grammar
from . import utilities
from .constants import DiceExtreme
from .exceptions import DiceBaseException, DiceException, DiceFatalException

__all__ = [
    "roll",
    "roll_min",
    "roll_max",
    "elements",
    "grammar",
    "utilities",
    "command",
    "DiceBaseException",
    "DiceException",
    "DiceFatalException",
    "DiceExtreme",
]
__author__ = "Sam Clements <sam@borntyping.co.uk>, " "Caleb Johnson <me@calebj.io>"
__version__ = "3.1.2"


def roll(string, **kwargs):
    """Parses and evaluates a dice expression"""
    return _roll(string, **kwargs)


def roll_min(string, **kwargs):
    """Parses and evaluates the minimum of a dice expression"""
    return _roll(string, force_extreme=DiceExtreme.EXTREME_MIN, **kwargs)


def roll_max(string, **kwargs):
    """Parses and evaluates the maximum of a dice expression"""
    return _roll(string, force_extreme=DiceExtreme.EXTREME_MAX, **kwargs)


def parse_expression(string):
    return grammar.expression.parseString(string, parseAll=True)


def _roll(string, single=True, raw=False, return_kwargs=False, **kwargs):
    try:
        ast = parse_expression(string)
        elements = list(ast)

        if not raw:
            elements = [element.evaluate_cached(**kwargs) for element in elements]

        if single:
            elements = utilities.single(elements)

        if return_kwargs:
            return elements, kwargs

        return elements
    except ParseBaseException as e:
        raise DiceBaseException.from_other(e)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (c) Camille Scott, 2020
# File   : zhistory.py
# License: MIT
# Author : Camille Scott <camille.scott.w@gmail.com>
# Date   : 22.02.2021

import typing

import discord
from discord.ext import commands

from ..database import fetch_guild_db
from ..logging import LoggingMixin
from ..utils import __time_format__


class HistoryCommands(commands.Cog, LoggingMixin):

    def __init__(self, bot, db):
        self.bot = bot
        self.db = db 

        super().__init__()

    @commands.command(name='hist', aliases=['history'])
    @fetch_guild_db
    async def history(self, ctx, member: typing.Optional[discord.Member],
                               max_rolls: typing.Optional[int] = 5):
        '''
        Display the roll history for the given member, or for the server
        if a member is not specified.
        '''

        records = [r async for r in ctx.guild_db.get_rolls(max_rolls=max_rolls)]

        def format(row):
            name = row.get('member_nick', None)
            if name is None:
                name = row.get('member_name', row.get('member_nick', 'Unknown'))
            timestamp = row['time'].strftime(__time_format__)
            result = f'{name} @ `{timestamp}`: `{row["roll"]:20}` ⟿  `{row["result"]}`'
            if row['tag']:
                result += f'# {row["tag"]}'
            return result

        msg = '\n'.join((format(record) for record in records))
        await ctx.send(f'**Roll :game_die: History**:\n{msg}')





from __future__ import absolute_import

from dice.constants import DiceExtreme
from dice.exceptions import DiceException, DiceFatalException
import pickle
from pytest import raises
import random

from dice.elements import (
    Integer,
    Roll,
    WildRoll,
    Dice,
    FudgeDice,
    Total,
    MAX_ROLL_DICE,
    Element,
    RandomElement,
    WildDice,
)
from dice import roll, roll_min, roll_max


class TestElements(object):
    def test_integer(self):
        assert isinstance(Integer(1), int)

    def test_dice_construct(self):
        d = Dice(2, 6)
        assert d.amount == 2 and d.amount == 2 and d.sides == d.max_value == 6

    def test_dice_from_iterable(self):
        d = Dice.from_iterable((2, 6))
        assert d.amount == 2 and d.sides == 6

    def test_dice_from_string(self):
        d = Dice.from_string("2d6")
        assert type(d) is Dice
        assert d.amount == 2 and d.sides == 6

        w = Dice.from_string("2w6")
        assert type(w) is WildDice
        assert w.amount == 2 and w.sides == 6

        u = Dice.from_string("2u6")
        assert type(u) is FudgeDice
        assert u.amount == 2 and u.sides == 6

    def test_dice_format(self):
        amount, sides = 6, 6
        for sep, cls in RandomElement.DICE_MAP.items():
            d = cls(amount, sides)
            assert str(d) == "%i%s%i" % (amount, cls.SEPARATOR, sides)
            assert repr(d) == "%s(%i, %i)" % (cls.__name__, amount, sides)

    def test_roll(self):
        amount, sides = 6, 6
        assert len(Roll.roll(amount, 1, sides)) == amount

        rr = Roll.roll(amount, 1, sides)
        assert (1 * sides) <= sum(rr) <= (amount * sides)

        rr = roll("%id%i" % (amount, sides))
        r_min, r_max = rr.random_element.min_value, rr.random_element.max_value
        assert r_min <= rr.do_roll_single() <= r_max

    def test_list(self):
        assert roll("1, 2, 3") == [1, 2, 3]

    def test_special_rhs(self):
        assert roll("d%", raw=True).max_value == 100
        fudge = roll("dF", raw=True)
        assert fudge.min_value == -1 and fudge.max_value == 1

    def test_extreme_roll(self):
        assert roll_min("3d6") == [1] * 3
        assert roll_max("3d6") == [6] * 3

    def test_fudge(self):
        amount, _range = 6, 6
        fdice = FudgeDice(amount, _range)
        assert fdice.min_value == -_range and fdice.max_value == _range
        froll = fdice.evaluate()
        assert len(froll) == amount
        assert -(amount * _range) <= sum(froll) <= (amount * _range)

    def test_wild(self):
        amount, sides = 6, 6
        assert len(WildRoll.roll(amount, 1, sides)) >= amount
        rr = WildRoll.roll(amount, 1, sides)
        assert 0 <= sum(rr)

        assert WildRoll.roll(0, 1, sides) == []

        assert WildRoll.roll(1, 1, 1) == [1]

    def test_wild_success(self):
        while True:
            if len(WildRoll.roll(1, 1, 2)) > 1:
                break

    def test_wild_fail(self):
        while True:
            if WildRoll.roll(1, 1, 2) == [0]:
                break

    def test_wild_critfail(self):
        while True:
            if WildRoll.roll(3, 1, 2) == [0, 0, 0]:
                break


class TestErrors(object):
    def test_toomanydice(self):
        with raises(DiceFatalException):
            roll("%id6" % (MAX_ROLL_DICE + 1))

        with raises(DiceFatalException):
            roll("7d6", max_dice=6)

        with raises(ValueError):
            Roll.roll(6, 1, 6, max_dice=4)

    def test_negative_amount_extreme(self):
        with raises(ValueError):
            Roll.roll(-1, 1, 6, force_extreme=DiceExtreme.EXTREME_MAX)

    def test_roll_error(self):
        with raises(ValueError):
            Roll.roll(-1, 1, 6)

        with raises(ValueError):
            Roll.roll_single(4, 3)

        with raises(DiceFatalException):
            rolled = roll("6d6")
            rolled.do_roll_single(4, 3)

    def test_invalid_wrap(self):
        with raises(NotImplementedError):
            try:
                raise RuntimeError('blah')
            except Exception as e:
                raise DiceException.from_other(e)


class TestEvaluate(object):
    def test_cache(self):
        """Test that evaluation returns the same result on successive runs"""
        roll("6d(6d6)t")
        ast = Total(Dice(6, Dice(6, 6)))
        evals = [ast.evaluate_cached() for i in range(100)]
        assert len(set(evals)) == 1
        assert ast.evaluate_cached() is ast.evaluate_cached()

    def test_nocache(self):
        """Test that evaluation returns different result on successive runs"""
        ast = Total(Dice(6, Dice(6, 6)))
        evals = [ast.evaluate() for i in range(100)]
        assert len(set(evals)) > 1

    def test_multiargs(self):
        """Test that binary operators function properly when repeated"""
        assert roll("1d1+1d1+1d1") == 3
        assert roll("1d1-1d1-1d1") == -1
        assert roll("1d1*1d1*1d1") == 1
        assert roll("1d1/1d1/1d1") == 1


class TestRegisterDice(object):
    def test_reregister(self):
        class FooDice(RandomElement):
            SEPARATOR = "d"

        with raises(RuntimeError):
            RandomElement.register_dice(FooDice)

    def test_no_separator(self):
        class BarDice(RandomElement):
            pass

        with raises(TypeError):
            RandomElement.register_dice(BarDice)

    def test_not_random_element(self):
        class BazDice(Element):
            pass

        with raises(TypeError):
            RandomElement.register_dice(BazDice)


class TestSystemRandom(object):
    sysrandom = random.SystemRandom()

    # lowest, middle and highest operators use shuffle()
    def test_sysrandom_op(self):
        roll("6d6h1", random=self.sysrandom)

    def test_sysrandom_roll(self):
        roll("6d6", random=self.sysrandom)


class TestPickle(object):
    for expr in [
        '-d20',
        '4d6t',
        '+-(1,2,3)',
        '2d20h',
        '4d6h3s',
        '4dF - 2',
        '4*d%'
    ]:
        value = roll(expr, raw=True, single=False)
        pickled = pickle.dumps(value)
        clone = pickle.loads(pickled)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (c) Camille Scott, 2021
# File   : rolls.py
# License: MIT
# Author : Camille Scott <camille.scott.w@gmail.com>
# Date   : 22.02.2021

from dice import DiceBaseException
from discord.ext import commands

import logging
import re

from .database import ZardozDatabase
from .state import GameMode, MODE_DICE, MAX_DICE_PER_ROLL
from .utils import SUCCESS, FAILURE
from .dice import roll as roll_expr
from .dice.elements import Comparison, Dice
from .dice.exceptions import DiceBaseException
from .dice.utilities import single


log = logging.getLogger()

OPS = ['+', '-', '<=', '<', '>=', '>', '(', ')', '==', '.+', '.-', '|', r'#']
DELIMS = [r'\s']
SPLIT_PAT = '|'.join(list(map(re.escape, OPS)) + DELIMS)


def split_tokens(word):
    log.info(f'SPLIT "{word}" on: "{SPLIT_PAT}"')
    return [t for t in re.split(f'({SPLIT_PAT})', word)]


def filter_tokens(tokens):
    return [t for t in tokens if t and not re.match(r'\s', t)]


def tokenize_roll(cmd: str):
    """tokenize_roll.

    Parameters
    ----------
    cmd : str
        Raw command string to tokenize.
    """

    raw = []

    log.info(f'Tokenize: "{cmd}"')
    if isinstance(cmd, str):
        raw = split_tokens(cmd)
    else:
        for arg in cmd:
            raw.extend(split_tokens(arg))

    raw = filter_tokens(raw)

    tokens, tag = [], ''
    for i, token in enumerate(raw):
        if token.startswith('#'):
            tokens = raw[:i]
            tag = ' '.join(raw[i:])
            break
    if not tag:
        tokens = raw
    tag = tag.strip('# ')

    return tokens, tag


def expand_tokens(tokens, mode = None, variables = {}):
    
    if mode is None:
        mode = GameMode.DEFAULT

    if mode is not None and not isinstance(mode, GameMode):
        raise ValueError('mode must be of type GameMode')

    expanded = []

    for token in tokens:
        if token.isnumeric():
            expanded.append(token)
        elif token in OPS:
            expanded.append(token)
        elif token.startswith('$'):
            var = token.strip('$')
            if var in variables:
                val = variables[var]
                if not isinstance(val, int):
                    raise ValueError('Non-integer variable!')
                expanded.append(variables[var])
            else:
                log.error(f'Undefined var: {var}')
                raise ValueError(f'Sorry bruh, that variable isn\'t defined: `{var}`')
        else:
            if token == 'r':
                token = MODE_DICE[mode]

            expanded.append(token)

    return expanded


def parse_tokens(tokens, **kwargs):
    eval_expr = ' '.join((str(token) for token in tokens))
    try:
        return roll_expr(eval_expr, **kwargs), eval_expr
    except DiceBaseException as e:
        raise ValueError(e)


def roll_expression(roll_expr, variables = {}, require_tag=False,
                    game_mode=GameMode.DEFAULT, **kwargs):
    tokens, tag = tokenize_roll(roll_expr)
    expanded = expand_tokens(tokens,
                             mode = game_mode,
                             variables = variables)
    result, eval_expr = parse_tokens(expanded, **kwargs)

    return result, expanded, tag, eval_expr


def extract_rolls(elements, rolls):
    for e in elements:
        if isinstance(e, Dice):
            rolls.append(e)
        elif hasattr(e, 'tokens'):
            extract_rolls(e.tokens, rolls)


class RollHandler:

    def __init__(self, ctx, log, variables, roll,
                       require_tag=False, game_mode=GameMode.DEFAULT):

        log.info(f'Roll request: {roll}')

        self.ctx = ctx
        self.log = log
        self.game_mode = game_mode
        self.roll = roll

        self.tokens, self.tag = tokenize_roll(roll)
        if require_tag and not self.tag:
            raise ValueError('Add a tag, it\'s the polite thing to do.')
        self.log.info(f'Raw tokens: {self.tokens}')

        self.expanded = expand_tokens(self.tokens,
                                      mode = self.game_mode,
                                      variables = variables)
        self.log.info(f'Expanded tokens: {self.expanded}')
        
        result, self.expr = parse_tokens(self.expanded, single=False, raw=True)
        self.result = RollResult(self.expr, result)
        if self.result is None:
            log.error(f'Python parsing error: {eval_expr}.')
            raise ValueError(f'Your syntax was scintillating, but I couldn\'t parse it.')

        log.info(f'Handled roll: {str(self)}')

    async def add_to_db(self, db: ZardozDatabase):
        await db.add_roll(self.ctx.author.id, self.ctx.author.nick, self.ctx.author.name,
                          ' '.join(self.tokens), self.tag, self.expr)

    def msg(self):
        header = f':game_die: {self.ctx.author.mention}' + (f': *{self.tag}*' if self.tag else '')
        result = [f'***Request:***  `{" ".join(self.tokens)}`\n',
                  f'***Rolls:***  `{self.result.describe_rolls()}`\n',
                  f'***Result:***\n```{self.result.describe(mode=self.game_mode)}```']
        result = ''.join(result)
        msg = '\n'.join((header, result))

        return msg

    def __str__(self):
        ret = f'<RollHander'\
              f' cmd="{self.roll}"'\
              f' from={self.ctx.author}'\
              f' guild="{self.ctx.guild}"'\
              f' tokens={self.tokens}'\
              f' tag="{self.tag}"'\
              f' expanded={self.expanded}'\
              f' rolls="{self.result.describe_rolls()}"'\
              f' result={self.result}>'
        return ret


class QuietRollHandler(RollHandler):

    def msg(self):
        header = f'*:game_die: {self.tag}*' if self.tag else ''
        result =f'```{self.result.describe(mode=self.game_mode)}```'
        msg = f'{header}\n{result}'

        return msg


class SekretRollHandler(RollHandler):

    def msg(self):
        header = f':game_die: from **{self.ctx.author}** in **{self.ctx.guild}**: ' + (f'*{self.tag}*' if self.tag else '')
        result = [f'***Request:***  `{" ".join(self.tokens)}`\n',
                  f'***Rolls:***  `{self.result.describe_rolls()}`\n',
                  f'***Result:***\n```{self.result.describe(mode=self.game_mode)}```']
        result = ''.join(result)
        return '\n'.join((header, result))


class RerollHandler(RollHandler):

    def msg(self, reroll_target):
        header = f':game_die: {self.ctx.author.mention} rerolls {reroll_target}' + (f': *{self.tag}*' if self.tag else '')
        result = [f'***Request:***  `{" ".join(self.tokens)}`\n',
                  f'***Rolls:***  `{self.result.describe_rolls()}`\n',
                  f'***Result:***\n```{self.result.describe(mode=self.game_mode)}```']
        result = ''.join(result)
        msg = '\n'.join((header, result))

        return msg


class RollResult:

    def __init__(self, expr, roll):
        self.expr = expr
        self.roll_elements = roll
        try:
            roll = single([e.evaluate_cached(max_dice=MAX_DICE_PER_ROLL) for e in roll])
        except DiceBaseException as e:
            raise ValueError(e)

        if isinstance(roll, int):
            self.roll = [roll]
        else:
            self.roll = []
            for item in roll:
                if isinstance(item, Comparison):
                    self.roll.append(DiceComparison(item))
                else:
                    self.roll.append(DiceResult(item))

    def describe(self, mode='', **kwargs):
        dsc = '\n'.join((r.describe(mode=mode, expr=self.expr) for r in self.roll))
        return dsc if dsc else '0'

    def describe_rolls(self):
        rolls = []
        extract_rolls(self.roll_elements, rolls)
        dsc = []
        for roll in rolls:
            dsc.append(f'{roll} ⤳ {roll.evaluate_cached()}')
        return ', '.join(dsc)

    def __iter__(self):
        for r in self.roll:
            yield r

    def __str__(self):
        roll = self.roll
        if len(self.roll) == 1:
            roll = self.roll[0]
        return f'{{{self.expr} ⤳ {roll}}}'

    def __repr__(self):
        return f'RollList("{self.expr}", {self.roll})'


class DiceResult:

    def __init__(self, result):
        if not isinstance(result, int):
            raise ValueError(f'Must be scalar (got {result}).')
        self.result = result

    def describe(self, **kwargs):
        return f'{self.result}'

    def __int__(self):
        return self.result
    

class DiceComparison(Comparison):

    def __init__(self, other):
        super().__init__(other.left, other.right , other.delta, other.value, other.operator)

    def describe(self, mode = None, expr = '', **kwargs):
        if mode is not None and not isinstance(mode, GameMode):
            raise ValueError('mode must be of type GameMode')
        if mode is GameMode.RT and 'd100' in expr:
            degrees = self.delta // 10
            status = SUCCESS if self.value else FAILURE
            if degrees > 0:
                result = f'{status} {degrees}°'
            else:
                result = status
            return f'{self.left:4} {self.operator} {self.right:<4} ⤳ {result}'
        else:
            kind = f'{SUCCESS} by' if self.value else f'{FAILURE} by'
            return f'{self.left:4} {self.operator} {self.right:<4} ⤳ {kind} {self.delta}'


class SimpleRollConvert(commands.Converter):

    pat = r'^\d+d\d+t?$'

    async def convert(self, ctx, argument):
        match = re.match(self.pat, argument)
        log.info(f'match {argument}')

        if match is None:
            log.info(f'SimpleRollConvert: {argument} did not match')
            raise commands.BadArgument(f'{argument} is not a simple roll.')
        try:
            roll = dice.roll(argument)
        except:
            log.info(f'Failed to parse roll: {argument}')
            raise commands.BadArgument(f'Failed to parse roll: {argument}')
        else:
            log.info(f'SimpleRoll: {argument} ⤳ {roll}')
            return DieResult(argument, int(roll))




"""
Dice notation grammar

PyParsing is patched to make it easier to work with, by removing features
that get in the way of development and debugging. See the dice.utilities
module for more information.
"""

from __future__ import absolute_import, print_function, unicode_literals

from pyparsing import (
    CaselessLiteral,
    Forward,
    Literal,
    OneOrMore,
    Or,
    StringStart,
    StringEnd,
    Suppress,
    Word,
    nums,
    opAssoc,
)

from .elements import (
    Integer,
    Successes,
    Mul,
    Div,
    Modulo,
    Sub,
    Add,
    Identity,
    AddEvenSubOdd,
    Equals,
    LessThan,
    LessThanEqual,
    GreaterThan,
    GreaterThanEqual,
    Total,
    Sort,
    Lowest,
    Middle,
    Highest,
    Array,
    Extend,
    Explode,
    Reroll,
    ForceReroll,
    Negate,
    SuccessFail,
    RandomElement,
    Again,
)

from .utilities import patch_pyparsing, wrap_string

patch_pyparsing()


def operatorPrecedence(base, operators):
    """
    This re-implements pyparsing's operatorPrecedence function.

    It gets rid of a few annoying bugs, like always putting operators inside
    a Group, and matching the whole grammar with Forward first (there may
    actually be a reason for that, but I couldn't find it). It doesn't
    support trinary expressions, but they should be easy to add if it turns
    out I need them.
    """

    # The full expression, used to provide sub-expressions
    expression = Forward()

    # The initial expression
    last = base | Suppress("(") + expression + Suppress(")")

    def parse_operator(expr, arity, association, action=None, extra=None):
        return expr, arity, association, action, extra

    for op in operators:
        # Use a function to default action to None
        expr, arity, association, action, extra = parse_operator(*op)

        # Check that the arity is valid
        if arity < 1 or arity > 2:
            raise Exception("Arity must be unary (1) or binary (2)")

        if association not in (opAssoc.LEFT, opAssoc.RIGHT):
            raise Exception("Association must be LEFT or RIGHT")

        # This will contain the expression
        this = Forward()

        # Create an expression based on the association and arity
        if association is opAssoc.LEFT:
            new_last = (last | extra) if extra else last
            if arity == 1:
                operator_expression = new_last + OneOrMore(expr)
            elif arity == 2:
                operator_expression = last + OneOrMore(expr + new_last)
        elif association is opAssoc.RIGHT:
            new_this = (this | extra) if extra else this
            if arity == 1:
                operator_expression = expr + new_this
            # Currently no operator uses this, so marking it nocover for now
            elif arity == 2:  # nocover
                operator_expression = last + OneOrMore(new_this)  # nocover

        # Set the parse action for the operator
        if action is not None:
            operator_expression.setParseAction(action)

        this <<= operator_expression | last
        last = this

    # Set the full expression and return it
    expression <<= last
    return expression


# An integer value
integer = Word(nums)
integer.setParseAction(Integer.parse)
integer.setName("integer")

dice_separators = RandomElement.DICE_MAP.keys()
dice_element = Or(
    wrap_string(CaselessLiteral, x, suppress=False) for x in dice_separators
)
special = wrap_string(Literal, "%", suppress=False) | wrap_string(
    CaselessLiteral, "f", suppress=False
)

# An expression in dice notation
expression = (
    StringStart()
    + operatorPrecedence(
        integer,
        [
            (dice_element, 2, opAssoc.LEFT, RandomElement.parse, special),
            (dice_element, 1, opAssoc.RIGHT, RandomElement.parse_unary, special),
            (wrap_string(CaselessLiteral, "x"), 2, opAssoc.LEFT, Explode.parse),
            (wrap_string(CaselessLiteral, "x"), 1, opAssoc.LEFT, Explode.parse),
            (wrap_string(CaselessLiteral, "rr"), 2, opAssoc.LEFT, ForceReroll.parse),
            (wrap_string(CaselessLiteral, "rr"), 1, opAssoc.LEFT, ForceReroll.parse),
            (wrap_string(CaselessLiteral, "r"), 2, opAssoc.LEFT, Reroll.parse),
            (wrap_string(CaselessLiteral, "r"), 1, opAssoc.LEFT, Reroll.parse),
            (wrap_string(Word, "^hH", exact=1), 2, opAssoc.LEFT, Highest.parse),
            (wrap_string(Word, "^hH", exact=1), 1, opAssoc.LEFT, Highest.parse),
            (wrap_string(Word, "vlL", exact=1), 2, opAssoc.LEFT, Lowest.parse),
            (wrap_string(Word, "vlL", exact=1), 1, opAssoc.LEFT, Lowest.parse),
            (wrap_string(Word, "oOmM", exact=1), 2, opAssoc.LEFT, Middle.parse),
            (wrap_string(Word, "oOmM", exact=1), 1, opAssoc.LEFT, Middle.parse),
            (wrap_string(CaselessLiteral, "a"), 2, opAssoc.LEFT, Again.parse),
            (wrap_string(CaselessLiteral, "a"), 1, opAssoc.LEFT, Again.parse),
            (wrap_string(CaselessLiteral, "e"), 2, opAssoc.LEFT, Successes.parse),
            (wrap_string(CaselessLiteral, "f"), 2, opAssoc.LEFT, SuccessFail.parse),
            (wrap_string(CaselessLiteral, "t"), 1, opAssoc.LEFT, Total.parse),
            (wrap_string(CaselessLiteral, "s"), 1, opAssoc.LEFT, Sort.parse),
            (wrap_string(Literal, "+-"), 1, opAssoc.RIGHT, AddEvenSubOdd.parse),
            (wrap_string(Literal, "+"), 1, opAssoc.RIGHT, Identity.parse),
            (wrap_string(Literal, "-"), 1, opAssoc.RIGHT, Negate.parse),
            (wrap_string(Literal, "%"), 2, opAssoc.LEFT, Modulo.parse),
            (wrap_string(Literal, "/"), 2, opAssoc.LEFT, Div.parse),
            (wrap_string(Literal, "*"), 2, opAssoc.LEFT, Mul.parse),
            (wrap_string(Literal, "-"), 2, opAssoc.LEFT, Sub.parse),
            (wrap_string(Literal, "+"), 2, opAssoc.LEFT, Add.parse),
            (wrap_string(Literal, ","), 2, opAssoc.LEFT, Array.parse),
            (wrap_string(Literal, "|"), 2, opAssoc.LEFT, Extend.parse),
            (wrap_string(Literal, "<"), 2, opAssoc.LEFT, LessThan.parse),
            (wrap_string(Literal, "<="), 2, opAssoc.LEFT, LessThanEqual.parse),
            (wrap_string(Literal, ">"), 2, opAssoc.LEFT, GreaterThan.parse),
            (wrap_string(Literal, ">="), 2, opAssoc.LEFT, GreaterThanEqual.parse),
            (wrap_string(Literal, '=='), 2, opAssoc.LEFT, Equals.parse),
        ],
    )
    + StringEnd()
)
expression.setName("expression")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (c) Camille Scott, 2021
# File   : zsample.py
# License: MIT
# Author : Camille Scott <camille.scott.w@gmail.com>
# Date   : 19.03.2021

import random
import typing

import discord
from discord.ext import commands

from ..database import fetch_guild_db
from ..logging import LoggingMixin
from ..rolls import tokenize_roll
from ..utils import handle_http_exception


class SampleCommands(commands.Cog, LoggingMixin):

    def __init__(self, bot, db):
        self.bot = bot
        self.db = db

        super().__init__()

    @commands.command(name='sample')
    @fetch_guild_db
    @handle_http_exception
    async def sample(self, ctx, k: int, N: int, *, args=''):
        '''
        Sample integers from discreet distributions.
        '''
        try:
            if k < 1 or N < 1:
                raise ValueError('`k` and `N` must be greater than 1!')
            if k > N:
                raise ValueError('`k` (sample size) cannot be greater than `N`!')
            if N > 100:
                raise ValueError(f'`N`={N} too large! Try 100 or less dawg.')
        except ValueError as e:
            await ctx.message.reply(f'I dun wan it: {e}')
            return

        pop = list(range(1, N+1))
        sample = sorted(random.sample(pop, k))
        roll, tag = tokenize_roll(args)

        header = f':question: {ctx.author.mention}' + (f': *{tag}*' if tag else '')
        action = f'Sample **{k}** from *[1..{N}]*'
        result = f'***Result:***\n```{sample}```'
        msg = '\n'.join([header, action, result])

        await ctx.message.reply(msg)




from __future__ import absolute_import, unicode_literals

from . import elements
from .constants import VERBOSE_INDENT
import warnings
import pyparsing


def classname(obj):
    """Returns the name of an objects class"""
    return obj.__class__.__name__


def single(iterable):
    """Returns a single item if the iterable has only one item"""
    return iterable[0] if len(iterable) == 1 else iterable


def patch_pyparsing(packrat=True, arity=True):
    """Applies monkey-patches to pyparsing"""
    if packrat:
        enable_pyparsing_packrat()

    if arity:
        disable_pyparsing_arity_trimming()


def enable_pyparsing_packrat():
    """Enables pyparsing's packrat parsing, which is much faster for the type
    of parsing being done in this library"""
    warnings.warn("Enabled pyparsing packrat parsing", ImportWarning)
    pyparsing.ParserElement.enablePackrat()


def _trim_arity(func, maxargs=None):
    def wrapper(string, location, tokens):
        return func(string, location, tokens)

    return wrapper


def disable_pyparsing_arity_trimming():
    """When pyparsing encounters a TypeError when calling a parse action, it
    will keep trying the call the function with one less argument each time
    until it succeeds. This disables this functionality, as it catches
    TypeErrors raised by other functions and makes debugging those functions
    very hard to do."""
    warnings.warn("Disabled pyparsing arity trimming", ImportWarning)
    pyparsing._trim_arity = _trim_arity


def wrap_string(cls, *args, **kwargs):
    suppress = kwargs.pop("suppress", True)
    e = cls(*args, **kwargs)
    e.setParseAction(elements.String.parse)
    if suppress:
        return e.suppress()
    return e


def add_even_sub_odd(operator, operand):
    """Add even numbers, subtract odd ones. See http://1w6.org/w6 """
    try:
        for i, x in enumerate(operand):
            if x % 2:
                operand[i] = -x
        return operand
    except TypeError:
        if operand % 2:
            return -operand
        return operand


def dice_switch(amount, dice_type, kind="d"):
    kind = kind.lower()
    if len(kind) != 1:
        raise ValueError("Dice operator must be 1 letter", 1)

    if isinstance(dice_type, int) and int(dice_type) < 1:
        raise ValueError("Number of sides must be one or more", 2)
    elif isinstance(dice_type, str):
        dice_type = dice_type.lower()

    if dice_type == "f":
        if kind not in ("d", "u"):
            raise ValueError("can only use dF or uF", 2)
        return dice.elements.FudgeDice(amount, 1)
    elif kind not in elements.RandomElement.DICE_MAP:
        raise ValueError("unknown dice kind: %s" % kind, 1)

    random_element = elements.RandomElement.DICE_MAP[kind]

    if str(dice_type) == "%":
        dice_type = 100

    return random_element(amount, dice_type)


def verbose_print_op(element, depth=0):
    lines = [[depth, classname(element) + "("]]
    num_ops = len(element.original_operands)

    for i, e in enumerate(element.original_operands):
        newlines = verbose_print_sub(e, depth + 1)

        if len(newlines) > 1 or num_ops > 1:
            if i + 1 < num_ops:
                newlines[-1].append(",")
            lines.extend(newlines)
        else:
            lines[-1].extend(newlines[0][1:])

    closing = ") -> %s" % element.result

    if num_ops > 1 or len(lines) > 1 and lines[-1][0] < lines[-2][0]:
        lines.append([depth, closing])
    else:
        lines[-1].append(closing)

    return lines


def verbose_print_sub(element, indent=0, **kwargs):
    lines = []
    if isinstance(element, elements.Element) and not hasattr(element, "result"):
        element.evaluate_cached(**kwargs)

    if isinstance(element, elements.Operator):
        return verbose_print_op(element, indent)

    elif isinstance(element, elements.Dice):
        if any(
            not isinstance(op, (elements.Integer, int))
            for op in element.original_operands
        ):
            return verbose_print_op(element, indent)

        line = "roll %s -> %s" % (element, element.result)
    else:
        line = str(element)

    lines.append([indent, line])
    return lines


def verbose_print(element, **kwargs):
    lines = verbose_print_sub(element, **kwargs)
    lines = [(" " * (VERBOSE_INDENT * t[0]) + "".join(t[1:])) for t in lines]
    return "\n".join(lines)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (c) Camille Scott, 2020
# File   : database.py
# License: MIT
# Author : Camille Scott <camille.scott.w@gmail.com>
# Date   : 22.02.2021

import aiofiles
import aiosql
import aiosqlite

import asyncio
from datetime import datetime
import functools
import os
import sqlite3

from .logging import LoggingMixin
from .state import GameMode
from .utils import ZARDOZ_PKG_DIR


def load_sql_commands(*args, **kwargs):
    mode = kwargs.get('mode', 'aiosqlite')
    cmd_file = os.path.join(ZARDOZ_PKG_DIR, 'sql', 'commands.sql')
    cmds = aiosql.from_path(cmd_file, mode)
    return cmds


class DatabaseCache(LoggingMixin):

    def __init__(self, db_dir):
        self.cache = {}
        self.db_dir = db_dir
        db_dir.mkdir(parents=True, exist_ok=True)

        super().__init__()

        self.log.info(f'DatabaseCache using dir: {db_dir}')

    async def get_guild_db(self, guild_id: int):
        self.log.info(f'Fetch database for {guild_id}')
        db = self.cache.get(guild_id, None)
        if db is None:
            self.log.info(f'Database for {guild_id} not in cache.')
            db = await ZardozDatabase.build(self.db_dir, guild_id)
            self.cache[guild_id] = db
        return db

    async def close(self):
        self.log.info(f'Closing {len(self.cache)} database connections.')
        for guild_id, db in self.cache.items():
            await db.close()


class ZardozDatabase(LoggingMixin):

    def __init__(self, con, guild_id, async_mode=True):
        self.con = con
        self.con.row_factory = sqlite3.Row
        self.guild_id = guild_id

        if async_mode:
            self.cmds = load_sql_commands()
        else:
            self.cmds = load_sql_commands(mode='sqlite3')

        self._register_aiosql_commands(self.cmds)

        super().__init__()

    def _register_aiosql_commands(self, cmds):
        for cmd_name in cmds.available_queries:
            # get the method and bind the connection to it
            cmd_func = getattr(cmds, cmd_name)
            bound = functools.partial(cmd_func, self.con)

            # build the new method for ZardozDatabase
            @functools.wraps(bound)
            def wrapped_cmd(_self, *args, **kwargs):
                return bound(*args, **kwargs)

            # add it to ZardozDatabase instance
            new_name = f'{cmd_name}_cmd'
            setattr(self, new_name, bound)

    @classmethod
    async def build(cls, db_dir, guild_id):
        spec_file = os.path.join(ZARDOZ_PKG_DIR, 'sql', 'database.sql')
        async with aiofiles.open(spec_file, mode='r') as fp:
            spec_script = await fp.read()

        path = db_dir.joinpath(f'{guild_id}.db')
        con = await aiosqlite.connect(path)
        await con.executescript(spec_script)
        await con.commit()

        return cls(con, guild_id)

    @staticmethod
    def convert_time(timestamp):
        return datetime.fromtimestamp(timestamp).astimezone()
        
    async def close(self):
        await self.con.close()

    async def add_roll(self, member_id: int, member_nick: str, 
                             member_name: str, roll: str, tag: str, 
                             result: str):
        await self.insert_roll_cmd(member_id=member_id, member_nick=member_nick,
                                   member_name=member_name, roll=roll,
                                   tag=tag, result=result,
                                   time=datetime.now().timestamp())
        await self.con.commit()

    async def get_rolls(self, member_id=None, max_rolls=5, since=None):
        async with self.get_rolls_cursor_cmd(max_rolls=max_rolls) as cur:
            async for row in cur:
                result = dict(row)
                result['time'] = self.convert_time(result['time'])
                yield result

    async def get_last_user_roll(self, member_id: int):
        roll = await self.get_last_user_roll_cmd(member_id)
        if not roll:
            return None

        roll = dict(roll)
        roll['time'] = self.convert_time(roll['time'])
        return roll

    async def get_merged_vars(self, member_id: int):
        user_vars = await self.get_user_vars(member_id)
        guild_vars = await self.get_guild_vars()
        guild_vars.update(user_vars)
        return guild_vars

    async def set_user_var(self, member_id: int, var: str, val: int):
        await self.set_user_var_cmd(member_id=member_id, var=var, val=val)
        await self.con.commit()

    async def get_user_var(self, member_id: int, var: str):
        result = await self.get_user_var_cmd(member_id=member_id, var=var)
        return result['val'] if result else None
    
    async def get_user_vars(self, member_id: int):
        user_vars = {}
        async with self.get_user_vars_cursor_cmd(member_id=member_id) as cur:
            async for row in cur:
                user_vars[row['var']] = row['val']
        return user_vars

    async def del_user_var(self, member_id: int, var: str):
        await self.del_user_var_cmd(member_id=member_id, var=var)
        await self.con.commit()

    async def set_guild_var(self, member_id: int, var: str, val: int):
        await self.set_guild_var_cmd(member_id=member_id, var=var, val=val)
        await self.con.commit()

    async def get_guild_var(self, var: str):
        result = await self.get_guild_var_cmd(var=var)
        return result['val'] if result else None
    
    async def get_guild_vars(self):
        guild_vars = {}
        async with self.get_guild_vars_cursor_cmd() as cur:
            async for row in cur:
                guild_vars[row['var']] = row['val']
        return guild_vars

    async def del_guild_var(self, var: str):
        await self.del_guild_var_cmd(var=var)
        await self.con.commit()
    
    async def set_guild_mode(self, mode: GameMode):
        if not isinstance(mode, GameMode):
            try:
                mode = GameMode[mode]
            except KeyError:
                raise ValueError(f'{mode} is not a valid GameMode')
        await self.set_guild_var(0, 'MODE', mode)

    async def get_guild_mode(self):
        mode = await self.get_guild_var('MODE')
        if mode:
            return GameMode(mode)
        else:
            await self.set_guild_mode(GameMode.DEFAULT)
            return GameMode.DEFAULT


def fetch_guild_db(func):

    @functools.wraps(func)
    async def wrapper(self, ctx, *args, **kwargs):
        ctx.guild_db = await self.db.get_guild_db(ctx.guild.id)
        ctx.game_mode = await ctx.guild_db.get_guild_mode()
        ctx.variables = await ctx.guild_db.get_merged_vars(ctx.author.id)
        return await func(self, ctx, *args, **kwargs)

    return wrapper



# Python 2 doesn't nave enum, but this is close enough.
class DiceExtreme(object):
    EXTREME_MIN = "MIN"
    EXTREME_MAX = "MAX"


MAX_ROLL_DICE = 2 ** 10
MAX_EXPLOSIONS = 2 ** 8
VERBOSE_INDENT = 2



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['discord.py',
                'disputils',
                'rich',
                'dice',
                'pyyaml',
                'xdg',
                'aiofiles',
                'aiosql',
                'aiosqlite']

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest>=3', ]

setup(
    author="Camille Scott",
    author_email='cswel@ucdavis.edu',
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'zardoz = zardoz:main'
        ]
    },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Another dice bot for discord.",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='zardoz',
    name='zardoz',
    packages=find_packages(include=['zardoz', 'zardoz.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/camillescott/zardoz',
    version='1.0.0',
    zip_safe=False,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# (c) Camille Scott, 2020
# File   : utils.py
# License: MIT
# Author : Camille Scott <camille.scott.w@gmail.com>
# Date   : 22.02.2021

import argparse
from dataclasses import is_dataclass
from datetime import datetime
import functools
import os
from pathlib import Path
import random

import discord
from xdg import xdg_data_home


ZARDOZ_PKG_DIR = os.path.abspath(os.path.dirname(__file__))
__time_format__ = '%a %b %d %I:%M%p %Z'

FAILURE = 'F 🛇'
SUCCESS = 'S ✅'


def d10():
    return random.randint(1, 10)


def d100():
    return random.randint(1, 100)


def Nd10(n=1):
    return [random.randint(1,10) for _ in range(n)]


def Nd100(n=1):
    return [random.randint(1,100)  for _ in range(n)]


def default_database_dir(debug=False):
    if not debug:
        return xdg_data_home().joinpath('zardoz-bot', 'databases')
    else:
        return xdg_data_home().joinpath('zardoz-bot', 'debug', 'databases')


def default_log_file(debug=False):
    if not debug:
        return xdg_data_home().joinpath('zardoz-bot', 'bot.log')
    else:
        return xdg_data_home().joinpath('zardoz-bot', 'debug', 'bot.log')


def reverse_number(num: int):
    result = 0
    while (num > 0):
        result = (num * 10) + (num % 10)
        num = num // 10
    return result


def handle_http_exception(func):

    @functools.wraps(func)
    async def wrapper(self, ctx, *args, **kwargs):
        try:
            return await func(self, ctx, *args, **kwargs)
        except discord.HTTPException:
            await ctx.message.reply('Hey broh, that\'s a big response.'\
                                    ' Try dialing back your request,'\
                                    ' I\'m only human.')
            raise
    return wrapper

